use strict;

use Test::More qw(no_plan);

use Vcdiff;

## Since Vcdiff is just an abstract loader module, there's not much to
## test here. See lib/Vcdiff/Test.pm in this distribution for the shared
## test infrastructure that the backend distributions use.

ok(1, 'loaded');
